/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.realm;

import java.util.HashMap;
import java.util.Map;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.keycloak.representations.idm.RealmRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.keycloak.representations.idm.RolesRepresentation;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.operator.keycloak.realm.OzgCloudKeycloakRealmSpec.RealmRole;

@Mapper(unmappedTargetPolicy = ReportingPolicy.IGNORE, unmappedSourcePolicy = ReportingPolicy.IGNORE)
interface KeycloakRealmMapper {

	@Mapping(target = "displayName", source = "displayName")
	@Mapping(target = "enabled", constant = "true")
	@Mapping(target = "resetPasswordAllowed", constant = "true")
	@Mapping(target = "supportedLocales", source = ".", qualifiedByName = "supportedLocales")
	@Mapping(target = "defaultLocale", constant = "de")
	@Mapping(target = "internationalizationEnabled", constant = "true")
	@Mapping(target = "passwordPolicy", constant = "upperCase(1) and lowerCase(1) and length(8) and notUsername")
	@Mapping(target = "actionTokenGeneratedByUserLifespan", constant = "900")
	@Mapping(target = "smtpServer", source = "smtpServer", qualifiedByName = "smtpServer")
	@Mapping(target = "roles.realm", source = "realmRoles")
	public RealmRepresentation map(OzgCloudKeycloakRealmSpec realm);


	@Mapping(target = "name", source = "name")
    RoleRepresentation map(OzgCloudKeycloakRealmSpec.RealmRole role);

	@Named("supportedLocales")
	default Set<String> mapPassword(OzgCloudKeycloakRealmSpec spec) {
		return Set.of("de");
	}

	@Named("smtpServer")
	default Map<String, String> mapSmtpServer(OzgCloudKeycloakRealmSpec.KeycloakRealmSMTPServer server) {

		Map<String, String> smtpServer = new HashMap<>();
		if (server != null) {
			smtpServer.put("host", server.getHost());
			smtpServer.put("port", server.getPort());
			smtpServer.put("user", server.getUser());
			smtpServer.put("password", server.getPassword());
			smtpServer.put("starttls", server.getStarttls());
			smtpServer.put("auth", server.getAuth());
			smtpServer.put("from", server.getFrom());
			smtpServer.put("fromDisplayName", server.getFromDisplayName());
		}
		return smtpServer;

	}

}
